@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -316,6 +316,25 @@ public:
 
 /// An enum for the different categories that a diff tree node falls
 /// into, regarding the kind of changes it represents.
+///
+/// Note that if you add an enumerator to this enum, you need to
+/// update a few spots accordingly:
+///
+///   * update the ACCESS_CHANGE_CATEGORY enumerator (which is the
+///     last enumerator of this enum by OR-ing its initializer with
+///     the new enumerator.
+///
+///   * update the categorize_harmless_diff_node or
+///     categorize_harmful_diff_node function depending on if the new
+///     enumerator classifies diff nodes as harmless or harmful.
+///
+///   * update the get_default_harmless_categories_bitmap or
+///    get_default_harmful_categories_bitmap function as well, just
+///    like above.
+///
+///   * update the "operator<<(ostream& o, diff_category c)" streaming
+///     operator so that it can stream the new enumerator to a textual
+///     output stream.
 enum diff_category
 {
   /// This means the diff node does not carry any (meaningful) change,
@@ -394,14 +413,18 @@ enum diff_category
   /// cv-qualifier change.
   FN_RETURN_TYPE_CV_CHANGE_CATEGORY = 1 << 15,
 
+  /// A diff node in this category is for a variable which type holds
+  /// a cv-qualifier change.
+  VAR_TYPE_CV_CHANGE_CATEGORY = 1 << 16,
+
   /// A diff node in this category carries a change from void pointer
   /// to non-void pointer.
-  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 16,
+  VOID_PTR_TO_PTR_CHANGE_CATEGORY = 1 << 17,
 
   /// A diff node in this category carries a change in the size of the
   /// array type of a global variable, but the ELF size of the
   /// variable didn't change.
-  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 17,
+  BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY = 1 << 18,
   /// A special enumerator that is the logical 'or' all the
   /// enumerators above.
   ///
@@ -424,6 +447,7 @@ enum diff_category
   | FN_PARM_TYPE_TOP_CV_CHANGE_CATEGORY
   | FN_PARM_TYPE_CV_CHANGE_CATEGORY
   | FN_RETURN_TYPE_CV_CHANGE_CATEGORY
+  | VAR_TYPE_CV_CHANGE_CATEGORY
   | VOID_PTR_TO_PTR_CHANGE_CATEGORY
   | BENIGN_INFINITE_ARRAY_CHANGE_CATEGORY
 }; // enum diff_category
