@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2018 Red Hat, Inc.
+// Copyright (C) 2016-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -82,6 +82,16 @@ public:
   interned_string(const interned_string& o)
   {raw_ = o.raw_;}
 
+  /// Assignment operator.
+  ///
+  /// @param o the other instance to assign to the current one.
+  interned_string&
+  operator=(const interned_string& o)
+  {
+    raw_ = o.raw_;
+    return *this;
+  }
+
   /// Test if the current instance of @ref interned_string is empty.
   ///
   /// @return true iff the currentisntance of @ref interned_string is
