@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2018 Red Hat, Inc.
+// Copyright (C) 2016-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -254,6 +254,12 @@ public:
   void
   set_source_location_to_keep_regex_str(const string&);
 
+  const vector<string>&
+  get_changed_enumerator_names() const;
+
+  void
+  set_changed_enumerator_names(const vector<string>&);
+
   virtual bool
   suppresses_diff(const diff* diff) const;
 
