@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -941,7 +941,7 @@ is_mostly_distinct_diff(const diff *d)
 
   // At this point, if we are not looking at a type diff we must have
   // bailed out already.
-  assert(td);
+  ABG_ASSERT(td);
 
   type_base_sptr first = is_type(td->first_subject());
   type_base_sptr second = is_type(td->second_subject());
@@ -950,7 +950,7 @@ is_mostly_distinct_diff(const diff *d)
 						 /*peel_qualified_type=*/false);
   second = peel_typedef_pointer_or_reference_type(second,
 						  /*peel_qual_type=*/false);
-  assert(first && second);
+  ABG_ASSERT(first && second);
 
   return distinct_diff::entities_are_of_distinct_kinds(first, second);
 }
@@ -1122,6 +1122,14 @@ type_diff_has_cv_qual_change_only(const diff *type_dif)
   f = peel_qualified_type(f);
   s = peel_qualified_type(s);
 
+  // If f and s are arrays, note that they can differ only by the cv
+  // qualifier of the array element type.  That cv qualifier is not
+  // removed by peel_qualified_type.  So we need to test this case
+  // specifically.
+  if (array_type_def *f_a = is_array_type(f))
+    if (array_type_def *s_a = is_array_type(s))
+      return equals_modulo_cv_qualifier(f_a, s_a);
+
   return *f == *s;
 }
 
@@ -1152,8 +1160,9 @@ has_fn_parm_type_cv_qual_change(const diff* dif)
 /// Test if a function type or decl diff node carries a CV
 /// qualifier-only change on its return type.
 ///
-/// @param the dif to consider.  Note that if this is neither a
-/// function type nor decl diff node, the function returns false.
+/// @param dif the diff node to consider.  Note that if this is
+/// neither a function type nor decl diff node, the function returns
+/// false.
 ///
 /// @return true iff @p dif is a function decl or type diff node which
 /// carries a CV qualifier-only change on its return type.
@@ -1172,6 +1181,36 @@ has_fn_return_type_cv_qual_change(const diff* dif)
   return type_diff_has_cv_qual_change_only(return_type_diff);
 }
 
+/// Test if a variable diff node carries a CV qualifier change on its type.
+///
+/// @param dif the diff node to consider.  Note that if it's not of
+/// var_diff type, the function returns false.
+///
+/// @return true iff the @p dif carries a CV qualifier change on its
+/// type.
+static bool
+has_var_type_cv_qual_change(const diff* dif)
+{
+  const var_diff *var_dif = is_var_diff(dif);
+  if (!var_dif)
+    return false;
+
+  {
+    // Make sure the variable diff does carry a type change at least
+    change_kind ch_kind;
+    if (equals(*var_dif->first_var(), *var_dif->second_var(), &ch_kind))
+      return false;
+
+    if (!(ch_kind & LOCAL_TYPE_CHANGE_KIND || ch_kind & SUBTYPE_CHANGE_KIND))
+      return false;
+  }
+
+  diff *type_dif = var_dif->type_diff().get();
+  ABG_ASSERT(type_dif);
+
+  return type_diff_has_cv_qual_change_only(type_dif);
+}
+
 /// Test if a diff node carries a void* to pointer type change.
 ///
 /// Note that this function looks through typedef and qualifier types
@@ -1328,6 +1367,9 @@ categorize_harmless_diff_node(diff *d, bool pre)
       if (has_fn_return_type_cv_qual_change(d))
 	category |= FN_RETURN_TYPE_CV_CHANGE_CATEGORY;
 
+      if (has_var_type_cv_qual_change(d))
+	category |= VAR_TYPE_CV_CHANGE_CATEGORY;
+
       if (has_void_ptr_to_ptr_change(d))
 	category |= VOID_PTR_TO_PTR_CHANGE_CATEGORY;
 
