@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -20,6 +20,7 @@
 
 #include <cstring>
 
+#include "abg-fwd.h"
 #include "abg-internal.h"
 // <headers defining libabigail's API go under here>
 ABG_BEGIN_EXPORT_DECLARATIONS
@@ -182,7 +183,7 @@ compute_lcs(const char* str1, const char* str2, int &ses_len, string& lcs)
   for (unsigned i = 0; i < result.size(); ++i)
     {
       int x = result[i].x(), y = result[i].y();
-      assert(str1[x] == str2[y]);
+      ABG_ASSERT(str1[x] == str2[y]);
       lcs.push_back(str1[x]);
     }
 }
