@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -344,6 +344,14 @@ InOutSpec in_out_specs[] =
     "data/test-diff-dwarf/test45-anon-dm-change-report-0.txt",
     "output/test-diff-dwarf/test45-anon-dm-change-report-0.txt"
   },
+#ifdef HAVE_DW_LANG_Rust_enumerator
+  {
+    "data/test-diff-dwarf/test46-rust-libone.so",
+    "data/test-diff-dwarf/test46-rust-libtwo.so",
+    "data/test-diff-dwarf/test46-rust-report-0.txt",
+    "output/test-diff-dwarf/test46-rust-report-0.txt"
+  },
+#endif
   // This should be the last entry
   {NULL, NULL, NULL, NULL}
 };
@@ -385,16 +393,17 @@ main()
 	abigail::dwarf_reader::STATUS_UNKNOWN;
 
       environment_sptr env(new environment);
+      std::vector<char**> di_roots;
       abigail::corpus_sptr corp0 =
 	read_corpus_from_elf(in_elfv0_path,
-			     /*debug_info_root_path=*/0,
+			     /*debug_info_root_path=*/di_roots,
 			     env.get(),
 			     /*load_all_types=*/false,
 			     status);
 
       abigail::corpus_sptr corp1 =
 	read_corpus_from_elf(in_elfv1_path,
-			     /*debug_info_root_path=*/0,
+			     /*debug_info_root_path=*/di_roots,
 			     env.get(),
 			     /*load_all_types=*/false,
 			     status);
