@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -657,7 +657,7 @@ main()
   for (InOutSpec* s = in_out_specs; s->in_elfv0_path; ++s)
     {
       test_task_sptr t(new test_task(*s));
-      assert(task_queue.schedule_task(t));
+      ABG_ASSERT(task_queue.schedule_task(t));
     }
 
   /// Wait for all worker threads to finish their job, and wind down.
@@ -669,7 +669,7 @@ main()
   const vector<task_sptr>& completed_tasks =
     task_queue.get_completed_tasks();
 
-  assert(completed_tasks.size() == num_tests);
+  ABG_ASSERT(completed_tasks.size() == num_tests);
 
   for (vector<task_sptr>::const_iterator ti = completed_tasks.begin();
        ti != completed_tasks.end();
