@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -551,6 +551,46 @@ static InOutSpec in_out_specs[] =
     "data/test-diff-pkg/elfutils-libs-0.170-4.el7.x86_64-multiple-sym-vers-report-0.txt",
     "output/test-diff-pkg/elfutils-libs-0.170-4.el7.x86_64-multiple-sym-vers-report-0.txt"
   },
+  {
+    "data/test-diff-pkg/nss-3.23.0-1.0.fc23.x86_64.rpm",
+    "data/test-diff-pkg/nss-3.24.0-1.0.fc23.x86_64.rpm",
+    "--fail-no-dbg",
+    "",
+    "data/test-diff-pkg/nss-debuginfo-3.23.0-1.0.fc23.x86_64.rpm",
+    "data/test-diff-pkg/nss-debuginfo-3.24.0-1.0.fc23.x86_64.rpm",
+    "data/test-diff-pkg/nss-devel-3.23.0-1.0.fc23.x86_64.rpm",
+    "data/test-diff-pkg/nss-devel-3.24.0-1.0.fc23.x86_64.rpm",
+    "data/test-diff-pkg/nss-3.23.0-1.0.fc23.x86_64-report-0.txt",
+    "output/test-diff-pkg/nss-3.23.0-1.0.fc23.x86_64-report-0.txt"
+  },
+  {
+    "data/test-diff-pkg/GtkAda-gl-2.24.2-29.fc29.x86_64.rpm",
+    "data/test-diff-pkg/GtkAda-gl-2.24.2-30.fc30.x86_64.rpm",
+    "--fail-no-dbg",
+    "",
+    "data/test-diff-pkg/GtkAda-debuginfo-2.24.2-29.fc29.x86_64.rpm, "
+    "data/test-diff-pkg/GtkAda-gl-debuginfo-2.24.2-29.fc29.x86_64.rpm",
+    "data/test-diff-pkg/GtkAda-debuginfo-2.24.2-30.fc30.x86_64.rpm, "
+    "data/test-diff-pkg/GtkAda-gl-debuginfo-2.24.2-30.fc30.x86_64.rpm",
+    "data/test-diff-pkg/GtkAda-devel-2.24.2-29.fc29.x86_64.rpm",
+    "data/test-diff-pkg/GtkAda-devel-2.24.2-30.fc30.x86_64.rpm",
+    "data/test-diff-pkg/GtkAda-gl-2.24.2-29.fc29.x86_64--2.24.2-30.fc30.x86_64-report-0.txt",
+    "output/test-diff-pkg/GtkAda-gl-2.24.2-29.fc29.x86_64--2.24.2-30.fc30.x86_64-report-0.txt"
+  },
+  {
+    "data/test-diff-pkg/netcdf-fortran-mpich-4.4.4-10.fc29.x86_64.rpm",
+    "data/test-diff-pkg/netcdf-fortran-mpich-4.4.4-11.fc30.x86_64.rpm",
+    "--fail-no-dbg",
+    "",
+    "data/test-diff-pkg/netcdf-fortran-mpich-debuginfo-4.4.4-10.fc29.x86_64.rpm,"
+    "data/test-diff-pkg/netcdf-fortran-debuginfo-4.4.4-10.fc29.x86_64.rpm",
+    "data/test-diff-pkg/netcdf-fortran-mpich-debuginfo-4.4.4-11.fc30.x86_64.rpm,"
+    "data/test-diff-pkg/netcdf-fortran-debuginfo-4.4.4-11.fc30.x86_64.rpm",
+    "data/test-diff-pkg/netcdf-fortran-mpich-devel-4.4.4-10.fc29.x86_64.rpm",
+    "data/test-diff-pkg/netcdf-fortran-mpich-devel-4.4.4-11.fc30.x86_64.rpm",
+    "data/test-diff-pkg/netcdf-fortran-mpich-4.4.4-10.fc29.x86_64-4.4.4-11.fc30.x86_64-report-0.txt",
+    "output/test-diff-pkg/netcdf-fortran-mpich-4.4.4-10.fc29.x86_64-4.4.4-11.fc30.x86_64-report-0.txt"
+  },
 #endif //WITH_RPM
 
 #ifdef WITH_DEB
@@ -621,7 +661,7 @@ struct test_task : public abigail::workers::task
 	vector<string> debug_info_pkg_paths;
 	split_string(spec.first_in_debug_package_path, ", ",
 		     debug_info_pkg_paths);
-	assert(!debug_info_pkg_paths.empty());
+	ABG_ASSERT(!debug_info_pkg_paths.empty());
 	for (vector<string>::const_iterator s = debug_info_pkg_paths.begin();
 	     s != debug_info_pkg_paths.end();
 	     ++s)
@@ -638,7 +678,7 @@ struct test_task : public abigail::workers::task
 	split_string(spec.second_in_debug_package_path,
 		     ", ",
 		     debug_info_pkg_paths);
-	assert(!debug_info_pkg_paths.empty());
+	ABG_ASSERT(!debug_info_pkg_paths.empty());
 	for (vector<string>::const_iterator s = debug_info_pkg_paths.begin();
 	     s != debug_info_pkg_paths.end();
 	     ++s)
@@ -750,7 +790,7 @@ main()
   for (InOutSpec *s = in_out_specs; s->first_in_package_path; ++s)
     {
       test_task_sptr t(new test_task(*s));
-      assert(task_queue.schedule_task(t));
+      ABG_ASSERT(task_queue.schedule_task(t));
     }
 
   /// Wait for all worker threads to finish their job, and wind down.
@@ -761,7 +801,7 @@ main()
 
   const vector<task_sptr>& completed_tasks =
     task_queue.get_completed_tasks();
-  assert(completed_tasks.size() == num_tests);
+  ABG_ASSERT(completed_tasks.size() == num_tests);
 
   for (vector<task_sptr>::const_iterator ti = completed_tasks.begin();
        ti != completed_tasks.end();
