@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -86,6 +86,7 @@ struct options
   bool			no_arch;
   bool			no_corpus;
   bool			leaf_changes_only;
+  bool			fail_no_debug_info;
   bool			show_hexadecimal_values;
   bool			show_offsets_sizes_in_bits;
   bool			show_relative_offset_changes;
@@ -110,8 +111,10 @@ struct options
   bool			dump_diff_tree;
   bool			show_stats;
   bool			do_log;
-  shared_ptr<char>	di_root_path1;
-  shared_ptr<char>	di_root_path2;
+  vector<char*> di_root_paths1;
+  vector<char*> di_root_paths2;
+  vector<char**> prepared_di_root_paths1;
+  vector<char**> prepared_di_root_paths2;
 
   options()
     : display_usage(),
@@ -123,6 +126,7 @@ struct options
       no_arch(),
       no_corpus(),
       leaf_changes_only(),
+      fail_no_debug_info(),
       show_hexadecimal_values(),
       show_offsets_sizes_in_bits(true),
       show_relative_offset_changes(true),
@@ -148,6 +152,22 @@ struct options
       show_stats(),
       do_log()
   {}
+
+  ~options()
+  {
+    for (vector<char*>::iterator i = di_root_paths1.begin();
+	 i != di_root_paths1.end();
+	 ++i)
+      free(*i);
+
+    for (vector<char*>::iterator i = di_root_paths2.begin();
+	 i != di_root_paths2.end();
+	 ++i)
+      free(*i);
+
+    prepared_di_root_paths1.clear();
+    prepared_di_root_paths2.clear();
+  }
 };//end struct options;
 
 static void
@@ -174,6 +194,7 @@ display_usage(const string& prog_name, ostream& out)
        "default suppression specification\n"
     << " --no-architecture  do not take architecture in account\n"
     << " --no-corpus-path  do not take the path to the corpora into account\n"
+    << " --fail-no-debug-info  bail out if no debug info was found\n"
     << " --leaf-changes-only|-l  only show leaf changes, "
     "so no change impact analysis\n"
     << " --deleted-fns  display deleted public functions\n"
@@ -260,8 +281,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	    }
 	  // elfutils wants the root path to the debug info to be
 	  // absolute.
-	  opts.di_root_path1 =
-	    abigail::tools_utils::make_path_absolute(argv[j]);
+	  opts.di_root_paths1.push_back
+	    (abigail::tools_utils::make_path_absolute_to_be_freed(argv[j]));
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--debug-info-dir2")
@@ -276,8 +297,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	    }
 	  // elfutils wants the root path to the debug info to be
 	  // absolute.
-	  opts.di_root_path2 =
-	    abigail::tools_utils::make_path_absolute(argv[j]);
+	  opts.di_root_paths2.push_back
+	    (abigail::tools_utils::make_path_absolute_to_be_freed(argv[j]));
 	  ++i;
 	}
       else if (!strcmp(argv[i], "--headers-dir1")
@@ -337,6 +358,8 @@ parse_command_line(int argc, char* argv[], options& opts)
 	opts.no_arch = true;
       else if (!strcmp(argv[i], "--no-corpus-path"))
 	opts.no_corpus = true;
+      else if (!strcmp(argv[i], "--fail-no-debug-info"))
+	opts.fail_no_debug_info = true;
       else if (!strcmp(argv[i], "--leaf-changes-only")
 	       ||!strcmp(argv[i], "-l"))
 	opts.leaf_changes_only = true;
@@ -697,10 +720,12 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
        ++i)
     read_suppressions(*i, supprs);
 
-  if (!opts.headers_dir1.empty())
+  if (read_context_get_path(read_ctxt) == opts.file1
+      && !opts.headers_dir1.empty())
     {
       // Generate suppression specification to avoid showing ABI
-      // changes on types that are not defined in public headers.
+      // changes on types that are not defined in public headers for
+      // the first binary.
       //
       // As these suppression specifications are applied during the
       // corpus loading, they are going to be dropped from the
@@ -715,10 +740,12 @@ set_suppressions(ReadContextType& read_ctxt, const options& opts)
 	}
     }
 
-  if (!opts.headers_dir2.empty())
+  if (read_context_get_path(read_ctxt) == opts.file2
+      && !opts.headers_dir2.empty())
     {
       // Generate suppression specification to avoid showing ABI
-      // changes on types that are not defined in public headers.
+      // changes on types that are not defined in public headers for
+      // the second binary.
       //
       // As these suppression specifications are applied during the
       // corpus loading, they are going to be dropped from the
@@ -785,15 +812,30 @@ set_corpus_keep_drop_regex_patterns(options& opts, corpus_sptr c)
 ///
 /// @param ctxt the diff context to consider.
 static void
-adjust_diff_context_for_kmidiff(diff_context_sptr &ctxt)
+adjust_diff_context_for_kmidiff(diff_context &ctxt)
 {
-  ctxt->show_linkage_names(false);
-  ctxt->show_added_fns(false);
-  ctxt->show_added_vars(false);
-  ctxt->show_added_symbols_unreferenced_by_debug_info
+  ctxt.show_linkage_names(false);
+  ctxt.show_added_fns(false);
+  ctxt.show_added_vars(false);
+  ctxt.show_added_symbols_unreferenced_by_debug_info
     (false);
 }
 
+/// Convert options::di_root_paths{1,2} into
+/// options::prepared_di_root_paths{1,2} which is the suitable type
+/// format that the dwarf_reader expects.
+///
+/// @param o the options to consider.
+static void
+prepare_di_root_paths(options& o)
+{
+  abigail::tools_utils::convert_char_stars_to_char_star_stars
+    (o.di_root_paths1, o.prepared_di_root_paths1);
+
+  abigail::tools_utils::convert_char_stars_to_char_star_stars
+    (o.di_root_paths2, o.prepared_di_root_paths2);
+}
+
 /// Emit an appropriate error message if necessary, given an error
 /// code.
 ///
@@ -827,46 +869,64 @@ static abigail::tools_utils::abidiff_status
 handle_error(abigail::dwarf_reader::status status_code,
 	     const abigail::dwarf_reader::read_context* ctxt,
 	     const string& prog_name,
-	     const string& input_file_name,
-	     char** debug_info_dir1,
-	     char** debug_info_dir2)
+	     const options& opts)
 {
   if (!(status_code & abigail::dwarf_reader::STATUS_OK))
     {
       emit_prefix(prog_name, cerr)
-	<< "failed to read input file " << input_file_name << "\n";
+	<< "failed to read input file " << opts.file1 << "\n";
 
       if (status_code & abigail::dwarf_reader::STATUS_DEBUG_INFO_NOT_FOUND)
 	{
 	  emit_prefix(prog_name, cerr) <<
 	    "could not find the debug info\n";
-	  if (debug_info_dir1)
-	    {
-	      if (*debug_info_dir1 == 0)
+	  {
+	    if (opts.prepared_di_root_paths1.empty() == 0)
+	      emit_prefix(prog_name, cerr)
+		<< "Maybe you should consider using the "
+		"--debug-info-dir1 option to tell me about the "
+		"root directory of the debuginfo? "
+		"(e.g, --debug-info-dir1 /usr/lib/debug)\n";
+	    else
+	      {
 		emit_prefix(prog_name, cerr)
-		  << "Maybe you should consider using the "
-		  "--debug-info-dir1 option to tell me about the "
-		  "root directory of the debuginfo? "
-		  "(e.g, --debug-info-dir1 /usr/lib/debug)\n";
-	      else
-		emit_prefix(prog_name, cerr)
-		  << "Maybe the root path to the debug information '"
-		  << *debug_info_dir1 << "' is wrong?\n";
-	    }
+		  << "Maybe the root path to the debug information '";
+		for (vector<char**>::const_iterator i
+		       = opts.prepared_di_root_paths1.begin();
+		     i != opts.prepared_di_root_paths1.end();
+		     ++i)
+		  {
+		    if (i != opts.prepared_di_root_paths1.end())
+		      cerr << ", ";
+		    cerr << **i;
+		  }
+		cerr << "' is wrong?\n";
+	      }
+	  }
 
-	  if (debug_info_dir2)
-	    {
-	      if (*debug_info_dir2 == 0)
+	  {
+	    if (opts.prepared_di_root_paths2.empty())
+	      emit_prefix(prog_name, cerr)
+		<< "Maybe you should consider using the "
+		"--debug-info-dir2 option to tell me about the "
+		"root directory of the debuginfo? "
+		"(e.g, --debug-info-dir2 /usr/lib/debug)\n";
+	    else
+	      {
 		emit_prefix(prog_name, cerr)
-		  << "Maybe you should consider using the "
-		  "--debug-info-dir2 option to tell me about the "
-		  "root directory of the debuginfo? "
-		  "(e.g, --debug-info-dir2 /usr/lib/debug)\n";
-	      else
-		emit_prefix(prog_name, cerr)
-		  << "Maybe the root path to the debug information '"
-		  << *debug_info_dir2 << "' is wrong?\n";
-	    }
+		  << "Maybe the root path to the debug information '";
+		for (vector<char**>::const_iterator i
+		       = opts.prepared_di_root_paths2.begin();
+		     i != opts.prepared_di_root_paths2.end();
+		     ++i)
+		  {
+		    if (i != opts.prepared_di_root_paths2.end())
+		      cerr << ", ";
+		    cerr << **i;
+		  }
+		  cerr << "' is wrong?\n";
+	      }
+	  }
 	}
 
       if (status_code & abigail::dwarf_reader::STATUS_ALT_DEBUG_INFO_NOT_FOUND)
@@ -887,7 +947,7 @@ handle_error(abigail::dwarf_reader::status status_code,
       if (status_code & abigail::dwarf_reader::STATUS_NO_SYMBOLS_FOUND)
 	emit_prefix(prog_name, cerr)
 	  << "could not find the ELF symbols in the file '"
-	  << input_file_name
+	  << opts.file1
 	  << "'\n";
 
       return abigail::tools_utils::ABIDIFF_ERROR;
@@ -934,6 +994,8 @@ main(int argc, char* argv[])
       return 0;
     }
 
+  prepare_di_root_paths(opts);
+
   if (!maybe_check_suppression_files(opts))
     return (abigail::tools_utils::ABIDIFF_USAGE_ERROR
 	    | abigail::tools_utils::ABIDIFF_ERROR);
@@ -972,7 +1034,6 @@ main(int argc, char* argv[])
 	c2_status = abigail::dwarf_reader::STATUS_OK;
       corpus_sptr c1, c2;
       corpus_group_sptr g1, g2;
-      char *di_dir1 = 0, *di_dir2 = 0;
       bool files_suppressed = false;
 
       diff_context_sptr ctxt(new diff_context);
@@ -1001,23 +1062,22 @@ main(int argc, char* argv[])
 	case abigail::tools_utils::FILE_TYPE_ELF:
 	case abigail::tools_utils::FILE_TYPE_AR:
 	  {
-	    di_dir1 = opts.di_root_path1.get();
 	    abigail::dwarf_reader::read_context_sptr ctxt =
-	      abigail::dwarf_reader::create_read_context(opts.file1,
-							 &di_dir1, env.get(),
-							 /*readalltypes*/false,
-							 opts.linux_kernel_mode);
+	      abigail::dwarf_reader::create_read_context
+	      (opts.file1, opts.prepared_di_root_paths1, env.get(),
+	       /*readalltypes*/false, opts.linux_kernel_mode);
 	    assert(ctxt);
 
 	    abigail::dwarf_reader::set_show_stats(*ctxt, opts.show_stats);
 	    set_suppressions(*ctxt, opts);
 	    abigail::dwarf_reader::set_do_log(*ctxt, opts.do_log);
 	    c1 = abigail::dwarf_reader::read_corpus_from_elf(*ctxt, c1_status);
-	    if (!c1)
+	    if (!c1
+		|| (opts.fail_no_debug_info
+		    && (c1_status & STATUS_ALT_DEBUG_INFO_NOT_FOUND)
+		    && (c1_status & STATUS_DEBUG_INFO_NOT_FOUND)))
 	      return handle_error(c1_status, ctxt.get(),
-				  argv[0], opts.file1,
-				  &di_dir1,
-				  /*debug_info_dir2=*/0);
+				  argv[0], opts);
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_XML_CORPUS:
@@ -1030,9 +1090,7 @@ main(int argc, char* argv[])
 	    c1 = abigail::xml_reader::read_corpus_from_input(*ctxt);
 	    if (!c1)
 	      return handle_error(c1_status, /*ctxt=*/0,
-				  argv[0], opts.file1,
-				  /*debug_info_dir1=*/0,
-				  /*debug_info_dir2=*/0);
+				  argv[0], opts);
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP:
@@ -1045,19 +1103,14 @@ main(int argc, char* argv[])
 	    g1 = abigail::xml_reader::read_corpus_group_from_input(*ctxt);
 	    if (!g1)
 	      return handle_error(c1_status, /*ctxt=*/0,
-				  argv[0], opts.file1,
-				  /*debug_info_dir1=*/0,
-				  /*debug_info_dir2=*/0);
+				  argv[0], opts);
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_ZIP_CORPUS:
 #ifdef WITH_ZIP_ARCHIVE
 	  c1 = abigail::xml_reader::read_corpus_from_file(opts.file1);
 	  if (!c1)
-	    return handle_error(c1_status, /*ctxt=*/0,
-				argv[0], opts.file1,
-				/*debug_info_dir1=*/0,
-				/*debug_info_dir2=*/0);
+	    return handle_error(c1_status, /*ctxt=*/0, argv[0], opts);
 #endif //WITH_ZIP_ARCHIVE
 	  break;
 	case abigail::tools_utils::FILE_TYPE_RPM:
@@ -1082,23 +1135,22 @@ main(int argc, char* argv[])
 	case abigail::tools_utils::FILE_TYPE_ELF:
 	case abigail::tools_utils::FILE_TYPE_AR:
 	  {
-	    di_dir2 = opts.di_root_path2.get();
 	    abigail::dwarf_reader::read_context_sptr ctxt =
-	      abigail::dwarf_reader::create_read_context(opts.file2,
-							 &di_dir2, env.get(),
-							 /*readalltypes=*/false,
-							 opts.linux_kernel_mode);
+	      abigail::dwarf_reader::create_read_context
+	      (opts.file2, opts.prepared_di_root_paths2, env.get(),
+	       /*readalltypes=*/false, opts.linux_kernel_mode);
 	    assert(ctxt);
 	    abigail::dwarf_reader::set_show_stats(*ctxt, opts.show_stats);
 	    abigail::dwarf_reader::set_do_log(*ctxt, opts.do_log);
 	    set_suppressions(*ctxt, opts);
 
 	    c2 = abigail::dwarf_reader::read_corpus_from_elf(*ctxt, c2_status);
-	    if (!c2)
-	      return handle_error(c2_status, ctxt.get(),
-				  argv[0], opts.file2,
-				  /*debug_info_dir1=*/0,
-				  /*debug_info_dir2=*/&di_dir2);
+	    if (!c2
+		|| (opts.fail_no_debug_info
+		    && (c2_status & STATUS_ALT_DEBUG_INFO_NOT_FOUND)
+		    && (c2_status & STATUS_DEBUG_INFO_NOT_FOUND)))
+	      return handle_error(c2_status, ctxt.get(), argv[0], opts);
+
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_XML_CORPUS:
@@ -1110,10 +1162,8 @@ main(int argc, char* argv[])
 	    set_suppressions(*ctxt, opts);
 	    c2 = abigail::xml_reader::read_corpus_from_input(*ctxt);
 	    if (!c2)
-	      return handle_error(c2_status, /*ctxt=*/0,
-				  argv[0], opts.file2,
-				  /*debug_info_dir1=*/0,
-				  /*debug_info_dir2=*/0);
+	      return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
+
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_XML_CORPUS_GROUP:
@@ -1125,20 +1175,14 @@ main(int argc, char* argv[])
 	    set_suppressions(*ctxt, opts);
 	    g2 = abigail::xml_reader::read_corpus_group_from_input(*ctxt);
 	    if (!g2)
-	      return handle_error(c2_status, /*ctxt=*/0,
-				  argv[0], opts.file2,
-				  /*debug_info_dir1=*/0,
-				  /*debug_info_dir2=*/0);
+	      return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
 	  }
 	  break;
 	case abigail::tools_utils::FILE_TYPE_ZIP_CORPUS:
 #ifdef WITH_ZIP_ARCHIVE
 	  c2 = abigail::xml_reader::read_corpus_from_file(opts.file2);
 	  if (!c2)
-	    return handle_error(c2_status, /*ctxt=*/0,
-				argv[0], opts.file2,
-				/*debug_info_dir1=*/0,
-				/*debug_info_dir2=*/0);
+	    return handle_error(c2_status, /*ctxt=*/0, argv[0], opts);
 #endif //WITH_ZIP_ARCHIVE
 	  break;
 	case abigail::tools_utils::FILE_TYPE_RPM:
@@ -1210,7 +1254,7 @@ main(int argc, char* argv[])
 	      return abigail::tools_utils::ABIDIFF_OK;
 	    }
 
-	  adjust_diff_context_for_kmidiff(ctxt);
+	  adjust_diff_context_for_kmidiff(*ctxt);
 	  corpus_diff_sptr diff = compute_diff(g1, g2, ctxt);
 
 	  if (diff->has_net_changes())
