@@ -1,6 +1,6 @@
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2018 Red Hat, Inc.
+// Copyright (C) 2013-2019 Red Hat, Inc.
 //
 // This file is part of the GNU Application Binary Interface Generic
 // Analysis and Instrumentation Library (libabigail).  This library is
@@ -326,10 +326,11 @@ main(int argc, char* argv[])
 	case abigail::tools_utils::FILE_TYPE_AR:
 	  {
 	    di_root_path = opts.di_root_path.get();
+	    vector<char**> di_roots;
+	    di_roots.push_back(&di_root_path);
 	    abigail::dwarf_reader::read_context_sptr ctxt =
 	      abigail::dwarf_reader::create_read_context(opts.file_path,
-							 &di_root_path,
-							 env.get(),
+							 di_roots, env.get(),
 							 /*load_all_types=*/false);
 	    assert(ctxt);
 	    set_suppressions(*ctxt, opts);
