@@ -1,21 +1,29 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    This file is part of Threading Building Blocks.
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+    Threading Building Blocks is free software; you can redistribute it
+    and/or modify it under the terms of the GNU General Public License
+    version 2 as published by the Free Software Foundation.
+
+    Threading Building Blocks is distributed in the hope that it will be
+    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
+    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with Threading Building Blocks; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+    As a special exception, you may use this file as part of a free software
+    library without restriction.  Specifically, if other files instantiate
+    templates or use macros or inline functions from this file, or you compile
+    this file and link it with other files to produce an executable, this
+    file does not by itself cause the resulting executable to be covered by
+    the GNU General Public License.  This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
 */
 
 #ifndef __TBB__concurrent_queue_impl_H
@@ -33,7 +41,6 @@
 #include "../tbb_exception.h"
 #include "../tbb_profiling.h"
 #include <new>
-#include <utility>
 
 #if !TBB_USE_EXCEPTIONS && _MSC_VER
     // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
@@ -58,6 +65,9 @@ template<typename T, typename A> class concurrent_queue;
 
 template<typename T, typename A> class concurrent_bounded_queue;
 
+namespace deprecated {
+template<typename T, typename A> class concurrent_queue;
+}
 #endif
 
 //! For internal use only.
@@ -73,7 +83,6 @@ typedef size_t ticket;
 template<typename T> class micro_queue ;
 template<typename T> class micro_queue_pop_finalizer ;
 template<typename T> class concurrent_queue_base_v3;
-template<typename T> struct concurrent_queue_rep;
 
 //! parts of concurrent_queue_rep that do not have references to micro_queue
 /**
@@ -144,9 +153,6 @@ private:
     The caller is expected to zero-initialize it. */
 template<typename T>
 class micro_queue : no_copy {
-public:
-    typedef void (*item_constructor_t)(T* location, const void* src);
-private:
     typedef concurrent_queue_rep_base::page page;
 
     //! Class used to ensure exception-safety of method "pop"
@@ -157,21 +163,18 @@ private:
         ~destroyer() {my_value.~T();}
     };
 
-    void copy_item( page& dst, size_t dindex, const void* src, item_constructor_t construct_item ) {
-        construct_item( &get_ref(dst, dindex), src );
+    void copy_item( page& dst, size_t index, const void* src ) {
+        new( &get_ref(dst,index) ) T(*static_cast<const T*>(src));
     }
 
-    void copy_item( page& dst, size_t dindex, const page& src, size_t sindex,
-        item_constructor_t construct_item )
-    {
-        T& src_item = get_ref( const_cast<page&>(src), sindex );
-        construct_item( &get_ref(dst, dindex), static_cast<const void*>(&src_item) );
+    void copy_item( page& dst, size_t dindex, const page& src, size_t sindex ) {
+        new( &get_ref(dst,dindex) ) T( get_ref(const_cast<page&>(src),sindex) );
     }
 
     void assign_and_destroy_item( void* dst, page& src, size_t index ) {
         T& from = get_ref(src,index);
         destroyer d(from);
-        *static_cast<T*>(dst) = tbb::internal::move( from );
+        *static_cast<T*>(dst) = from;
     }
 
     void spin_wait_until_my_turn( atomic<ticket>& counter, ticket k, concurrent_queue_rep_base& rb ) const ;
@@ -200,36 +203,31 @@ public:
 
     spin_mutex page_mutex;
 
-    void push( const void* item, ticket k, concurrent_queue_base_v3<T>& base,
-        item_constructor_t construct_item ) ;
+    void push( const void* item, ticket k, concurrent_queue_base_v3<T>& base ) ;
 
     bool pop( void* dst, ticket k, concurrent_queue_base_v3<T>& base ) ;
 
-    micro_queue& assign( const micro_queue& src, concurrent_queue_base_v3<T>& base,
-        item_constructor_t construct_item ) ;
+    micro_queue& assign( const micro_queue& src, concurrent_queue_base_v3<T>& base ) ;
 
-    page* make_copy( concurrent_queue_base_v3<T>& base, const page* src_page, size_t begin_in_page,
-        size_t end_in_page, ticket& g_index, item_constructor_t construct_item ) ;
+    page* make_copy( concurrent_queue_base_v3<T>& base, const page* src_page, size_t begin_in_page, size_t end_in_page, ticket& g_index ) ;
 
     void invalidate_page_and_rethrow( ticket k ) ;
 };
 
 template<typename T>
 void micro_queue<T>::spin_wait_until_my_turn( atomic<ticket>& counter, ticket k, concurrent_queue_rep_base& rb ) const {
-    for( atomic_backoff b(true);;b.pause() ) {
-        ticket c = counter;
-        if( c==k ) return;
-        else if( c&1 ) {
+    atomic_backoff backoff;
+    do {
+        backoff.pause();
+        if( counter&1 ) {
             ++rb.n_invalid_entries;
             throw_exception( eid_bad_last_alloc );
         }
-    }
+    } while( counter!=k ) ;
 }
 
 template<typename T>
-void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<T>& base,
-    item_constructor_t construct_item )
-{
+void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<T>& base ) {
     k &= -concurrent_queue_rep_base::n_queue;
     page* p = NULL;
     size_t index = modulo_power_of_two( k/concurrent_queue_rep_base::n_queue, base.my_rep->items_per_page);
@@ -245,7 +243,7 @@ void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<
         p->next = NULL;
     }
 
-    if( tail_counter != k ) spin_wait_until_my_turn( tail_counter, k, *base.my_rep );
+    if( tail_counter!=k ) spin_wait_until_my_turn( tail_counter, k, *base.my_rep );
     call_itt_notify(acquired, &tail_counter);
 
     if( p ) {
@@ -259,9 +257,8 @@ void micro_queue<T>::push( const void* item, ticket k, concurrent_queue_base_v3<
     } else {
         p = tail_page;
     }
-
     __TBB_TRY {
-        copy_item( *p, index, item, construct_item );
+        copy_item( *p, index, item );
         // If no exception was thrown, mark item as present.
         itt_hide_store_word(p->mask,  p->mask | uintptr_t(1)<<index);
         call_itt_notify(releasing, &tail_counter);
@@ -298,11 +295,10 @@ bool micro_queue<T>::pop( void* dst, ticket k, concurrent_queue_base_v3<T>& base
 }
 
 template<typename T>
-micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_queue_base_v3<T>& base,
-    item_constructor_t construct_item )
-{
+micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_queue_base_v3<T>& base ) {
     head_counter = src.head_counter;
     tail_counter = src.tail_counter;
+    page_mutex   = src.page_mutex;
 
     const page* srcp = src.head_page;
     if( is_valid_page(srcp) ) {
@@ -312,12 +308,12 @@ micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_qu
             size_t index = modulo_power_of_two( head_counter/concurrent_queue_rep_base::n_queue, base.my_rep->items_per_page );
             size_t end_in_first_page = (index+n_items<base.my_rep->items_per_page)?(index+n_items):base.my_rep->items_per_page;
 
-            head_page = make_copy( base, srcp, index, end_in_first_page, g_index, construct_item );
+            head_page = make_copy( base, srcp, index, end_in_first_page, g_index );
             page* cur_page = head_page;
 
             if( srcp != src.tail_page ) {
                 for( srcp = srcp->next; srcp!=src.tail_page; srcp=srcp->next ) {
-                    cur_page->next = make_copy( base, srcp, 0, base.my_rep->items_per_page, g_index, construct_item );
+                    cur_page->next = make_copy( base, srcp, 0, base.my_rep->items_per_page, g_index );
                     cur_page = cur_page->next;
                 }
 
@@ -325,7 +321,7 @@ micro_queue<T>& micro_queue<T>::assign( const micro_queue<T>& src, concurrent_qu
                 size_t last_index = modulo_power_of_two( tail_counter/concurrent_queue_rep_base::n_queue, base.my_rep->items_per_page );
                 if( last_index==0 ) last_index = base.my_rep->items_per_page;
 
-                cur_page->next = make_copy( base, srcp, 0, last_index, g_index, construct_item );
+                cur_page->next = make_copy( base, srcp, 0, last_index, g_index );
                 cur_page = cur_page->next;
             }
             tail_page = cur_page;
@@ -356,17 +352,14 @@ void micro_queue<T>::invalidate_page_and_rethrow( ticket k ) {
 }
 
 template<typename T>
-concurrent_queue_rep_base::page* micro_queue<T>::make_copy( concurrent_queue_base_v3<T>& base,
-    const concurrent_queue_rep_base::page* src_page, size_t begin_in_page, size_t end_in_page,
-    ticket& g_index, item_constructor_t construct_item )
-{
+concurrent_queue_rep_base::page* micro_queue<T>::make_copy( concurrent_queue_base_v3<T>& base, const concurrent_queue_rep_base::page* src_page, size_t begin_in_page, size_t end_in_page, ticket& g_index ) {
     concurrent_queue_page_allocator& pa = base;
     page* new_page = pa.allocate_page();
     new_page->next = NULL;
     new_page->mask = src_page->mask;
     for( ; begin_in_page!=end_in_page; ++begin_in_page, ++g_index )
         if( new_page->mask & uintptr_t(1)<<begin_in_page )
-            copy_item( *new_page, begin_in_page, *src_page, begin_in_page, construct_item );
+            copy_item( *new_page, begin_in_page, *src_page, begin_in_page );
     return new_page;
 }
 
@@ -434,7 +427,6 @@ struct concurrent_queue_rep : public concurrent_queue_rep_base {
  */
 template<typename T>
 class concurrent_queue_base_v3: public concurrent_queue_page_allocator {
-private:
     //! Internal representation
     concurrent_queue_rep<T>* my_rep;
 
@@ -448,7 +440,6 @@ protected:
 
 private:
     typedef typename micro_queue<T>::padded_page padded_page;
-    typedef typename micro_queue<T>::item_constructor_t item_constructor_t;
 
     /* override */ virtual page *allocate_page() {
         concurrent_queue_rep<T>& r = *my_rep;
@@ -481,10 +472,10 @@ protected:
     }
 
     //! Enqueue item at tail of queue
-    void internal_push( const void* src, item_constructor_t construct_item ) {
-         concurrent_queue_rep<T>& r = *my_rep;
-         ticket k = r.tail_counter++;
-         r.choose(k).push( src, k, *this, construct_item );
+    void internal_push( const void* src ) {
+        concurrent_queue_rep<T>& r = *my_rep;
+        ticket k = r.tail_counter++;
+        r.choose(k).push( src, k, *this );
     }
 
     //! Attempt to dequeue item from queue.
@@ -506,15 +497,8 @@ protected:
         throw_exception( eid_bad_alloc );
     }
 
-    //! copy or move internal representation
-    void assign( const concurrent_queue_base_v3& src, item_constructor_t construct_item ) ;
-
-#if __TBB_CPP11_RVALUE_REF_PRESENT
-    //! swap internal representation
-    void internal_swap( concurrent_queue_base_v3& src ) {
-        std::swap( my_rep, src.my_rep );
-    }
-#endif /* __TBB_CPP11_RVALUE_REF_PRESENT */
+    //! copy internal representation
+    void assign( const concurrent_queue_base_v3& src ) ;
 };
 
 template<typename T>
@@ -601,20 +585,18 @@ void concurrent_queue_base_v3<T>::internal_finish_clear() {
 }
 
 template<typename T>
-void concurrent_queue_base_v3<T>::assign( const concurrent_queue_base_v3& src,
-    item_constructor_t construct_item )
-{
+void concurrent_queue_base_v3<T>::assign( const concurrent_queue_base_v3& src ) {
     concurrent_queue_rep<T>& r = *my_rep;
     r.items_per_page = src.my_rep->items_per_page;
 
-    // copy concurrent_queue_rep data
+    // copy concurrent_queue_rep.
     r.head_counter = src.my_rep->head_counter;
     r.tail_counter = src.my_rep->tail_counter;
     r.n_invalid_entries = src.my_rep->n_invalid_entries;
 
-    // copy or move micro_queues
-    for( size_t i = 0; i < r.n_queue; ++i )
-        r.array[i].assign( src.my_rep->array[i], *this, construct_item);
+    // copy micro_queues
+    for( size_t i = 0; i<r.n_queue; ++i )
+        r.array[i].assign( src.my_rep->array[i], *this);
 
     __TBB_ASSERT( r.head_counter==src.my_rep->head_counter && r.tail_counter==src.my_rep->tail_counter,
             "the source concurrent queue should not be concurrently modified." );
@@ -761,7 +743,7 @@ class concurrent_queue_iterator: public concurrent_queue_iterator_base_v3<typena
     template<typename T, class A>
     friend class ::tbb::strict_ppl::concurrent_queue;
 #else
-public:
+public: // workaround for MSVC
 #endif
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator( const concurrent_queue_base_v3<Value>& queue ) :
@@ -832,7 +814,6 @@ template<typename Container, typename Value> class concurrent_queue_iterator;
 /** Type-independent portion of concurrent_queue.
     @ingroup containers */
 class concurrent_queue_base_v3: no_copy {
-private:
     //! Internal representation
     concurrent_queue_rep* my_rep;
 
@@ -857,8 +838,6 @@ protected:
     //! Size of an item
     size_t item_size;
 
-    enum copy_specifics { copy, move };
-
 #if __TBB_PROTECTED_NESTED_CLASS_BROKEN
 public:
 #endif
@@ -879,7 +858,7 @@ protected:
     __TBB_EXPORTED_METHOD concurrent_queue_base_v3( size_t item_size );
     virtual __TBB_EXPORTED_METHOD ~concurrent_queue_base_v3();
 
-    //! Enqueue item at tail of queue using copy operation
+    //! Enqueue item at tail of queue
     void __TBB_EXPORTED_METHOD internal_push( const void* src );
 
     //! Dequeue item from head of queue
@@ -888,7 +867,7 @@ protected:
     //! Abort all pending queue operations
     void __TBB_EXPORTED_METHOD internal_abort();
 
-    //! Attempt to enqueue item onto queue using copy operation
+    //! Attempt to enqueue item onto queue.
     bool __TBB_EXPORTED_METHOD internal_push_if_not_full( const void* src );
 
     //! Attempt to dequeue item from queue.
@@ -920,49 +899,10 @@ protected:
     //! copy internal representation
     void __TBB_EXPORTED_METHOD assign( const concurrent_queue_base_v3& src ) ;
 
-#if __TBB_CPP11_RVALUE_REF_PRESENT
-    //! swap queues
-    void internal_swap( concurrent_queue_base_v3& src ) {
-        std::swap( my_capacity, src.my_capacity );
-        std::swap( items_per_page, src.items_per_page );
-        std::swap( item_size, src.item_size );
-        std::swap( my_rep, src.my_rep );
-    }
-#endif /* __TBB_CPP11_RVALUE_REF_PRESENT */
-
-    //! Enqueues item at tail of queue using specified operation (copy or move)
-    void internal_insert_item( const void* src, copy_specifics op_type );
-
-    //! Attempts to enqueue at tail of queue using specified operation (copy or move)
-    bool internal_insert_if_not_full( const void* src, copy_specifics op_type );
-
-    //! Assigns one queue to another using specified operation (copy or move)
-    void internal_assign( const concurrent_queue_base_v3& src, copy_specifics op_type );
 private:
     virtual void copy_page_item( page& dst, size_t dindex, const page& src, size_t sindex ) = 0;
 };
 
-//! For internal use only.
-/** Backward compatible modification of concurrent_queue_base_v3
-    @ingroup containers */
-class concurrent_queue_base_v8: public concurrent_queue_base_v3 {
-protected:
-    concurrent_queue_base_v8( size_t item_sz ) : concurrent_queue_base_v3( item_sz ) {}
-
-    //! move items
-    void __TBB_EXPORTED_METHOD move_content( concurrent_queue_base_v8& src ) ;
-
-    //! Attempt to enqueue item onto queue using move operation
-    bool __TBB_EXPORTED_METHOD internal_push_move_if_not_full( const void* src );
-
-    //! Enqueue item at tail of queue using move operation
-    void __TBB_EXPORTED_METHOD internal_push_move( const void* src );
-private:
-    friend struct micro_queue;
-    virtual void move_page_item( page& dst, size_t dindex, const page& src, size_t sindex ) = 0;
-    virtual void move_item( page& dst, size_t index, const void* src ) = 0;
-};
-
 //! Type-independent portion of concurrent_queue_iterator.
 /** @ingroup containers */
 class concurrent_queue_iterator_base_v3 {
@@ -1018,10 +958,12 @@ class concurrent_queue_iterator: public concurrent_queue_iterator_base,
 #if !defined(_MSC_VER) || defined(__INTEL_COMPILER)
     template<typename T, class A>
     friend class ::tbb::concurrent_bounded_queue;
+
+    template<typename T, class A>
+    friend class ::tbb::deprecated::concurrent_queue;
 #else
 public: // workaround for MSVC
 #endif
-
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator( const concurrent_queue_base_v3& queue ) :
         concurrent_queue_iterator_base_v3(queue,__TBB_offsetof(concurrent_queue_base_v3::padded_page<Value>,last))
