@@ -688,24 +688,6 @@ button_callback (gpointer data)
 
 static UINT taskbar_created_msg = 0;
 static GSList *status_icons = NULL;
-static UINT status_icon_id = 0;
-
-static GtkStatusIcon *
-find_status_icon (UINT id)
-{
-  GSList *rover;
-
-  for (rover = status_icons; rover != NULL; rover = rover->next)
-    {
-      GtkStatusIcon *status_icon = GTK_STATUS_ICON (rover->data);
-      GtkStatusIconPrivate *priv = status_icon->priv;
-
-      if (priv->nid.uID == id)
-        return status_icon;
-    }
-
-  return NULL;
-}
 
 static LRESULT CALLBACK
 wndproc (HWND   hwnd,
@@ -723,7 +705,7 @@ wndproc (HWND   hwnd,
 	  GtkStatusIconPrivate *priv = status_icon->priv;
 
 	  priv->nid.hWnd = hwnd;
-	  priv->nid.uID = status_icon_id++;
+	  priv->nid.uID = GPOINTER_TO_UINT (status_icon);
 	  priv->nid.uCallbackMessage = WM_GTK_TRAY_NOTIFICATION;
 	  priv->nid.uFlags = NIF_MESSAGE;
 
@@ -767,7 +749,7 @@ wndproc (HWND   hwnd,
 	buttondown0:
 	  bc = g_new (ButtonCallbackData, 1);
 	  bc->event = (GdkEventButton *) gdk_event_new (GDK_BUTTON_PRESS);
-	  bc->status_icon = find_status_icon (wparam);
+	  bc->status_icon = GTK_STATUS_ICON (wparam);
 	  build_button_event (bc->status_icon->priv, bc->event, button);
 	  g_idle_add (button_callback, bc);
 	  break;
@@ -793,7 +775,7 @@ wndproc (HWND   hwnd,
 	buttonup0:
 	  bc = g_new (ButtonCallbackData, 1);
 	  bc->event = (GdkEventButton *) gdk_event_new (GDK_BUTTON_RELEASE);
-	  bc->status_icon = find_status_icon (wparam);
+	  bc->status_icon = GTK_STATUS_ICON (wparam);
 	  build_button_event (bc->status_icon->priv, bc->event, button);
 	  g_idle_add (button_callback, bc);
 	  break;
@@ -927,7 +909,7 @@ gtk_status_icon_init (GtkStatusIcon *status_icon)
   memset (&priv->nid, 0, sizeof (priv->nid));
 
   priv->nid.hWnd = create_tray_observer ();
-  priv->nid.uID = status_icon_id++;
+  priv->nid.uID = GPOINTER_TO_UINT (status_icon);
   priv->nid.uCallbackMessage = WM_GTK_TRAY_NOTIFICATION;
   priv->nid.uFlags = NIF_MESSAGE;
 
